%Copyright (c)  Yiming Ying, 2008%%%%  
%%%% determine the optimal process based on the largest free energy lower
%%%% bound%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



 



%Input variables

%Data--- D x G (data x gene)  matrix of expression values
%pro--- process vector
%no_reps---number of iterations in the EM algorithm
%m_0,v_0---model hyper-parameters for mean ($\mu_{gk}$)
%a_0,b_0----model hyper-parameters for precision ($\beta_{gk}$)
%for example, m_0=0,  v_0=0.1,  a_0=20.0,  b_0=0.05  
%%here we preprocess the data with mean 0 and std 5. 



 


%Outputs
%CMean_m---G x K matrix, optimal posterior mean parameters  
%CAlpha---D x K  matrix, with each row vector as optimal posterior Dirichelet parameters
%Cfr_pro--- free energy vector verus process vector 
%Cprocess---returned optimal process number

clear all;
tic;
Path = '/exports/gpfs/enxyy/code/matlab/data/brownwilms/';
load([Path, 'brownwilms.mat']);

Data = normalize_meanstd(data);

%%%%%%%%%%%%Initializing parameters
pro=[2:12];  %%%%%%%% vector of process
no_reps=100;
m_0=mean(mean(Data));
Alphazero=1;
v_0=1;
a_0=20;
b_0=0.05;
mdisplay=1;  %%%%%%%%%% display programme running
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  


Cfr_pro=[];  %%%  free energy vector   verus  process

 




for  K=pro(1):pro(end)
    
    
     Alpha_0=Alphazero*ones(1,K); 
  
    
 
        
    % display iteration number wrt process
   
   
   if mdisplay==1
       
       disp(['Process: [', num2str(K),'] Time:', num2str(toc),' ----------------------------------------']);
       v=sprintf('process: %d',K);
       disp(v);
   end
    
    
   
       
 [CMean_m,CAlpha,CFenergy]...
    =cvb_pro(Data,K,no_reps,Alpha_0,m_0,v_0,a_0,b_0,mdisplay);
  
 
  
 Cfr_pro=[Cfr_pro CFenergy];
 
 
 
 
 
    
end


[max_fr max_pro]=max(Cfr_pro);

Cprocess=max_pro+pro(1)-1;    %%%  optimal process based on free energy criterion 



%%%output the optimal mean and alpha of the optimal process 

 

[CMean_m,CAlpha,CFenergy]...
    =cvb_pro(Data,Cprocess,no_reps,Alphazero*ones(1,Cprocess),m_0,v_0,a_0,b_0,mdisplay);

 
%plot the free energy vector verus process vector

% figure; 
% plot( [pro(1):1:pro(end)], Cfr_pro,'*-'); axis([pro(1) pro(end) min(Cfr_pro)  0.98*max(Cfr_pro)]);
% xlabel('process');
% ylabel('free energy');


%plot the normalization of the optimal alpha (clusters)

CAlpha_norm=CAlpha./repmat(sum(CAlpha,2),1,Cprocess);

% figure;
% bar_plot(CAlpha_norm');


save([Path,'result_cvblpd_brownswilms.mat']);
 

