%%%% Example script to find optimal number of clusters, 
%Copyright (c)  Yiming Ying, 2008%%%%
%Input variables
%Data--- D x G (data x gene)  matrix of expression values
%pro--- process vector
%no_reps---number of iterations in the EM algorithm
%m_0,v_0---model hyper-parameters for mean ($\mu_{gk}$)
%a_0,b_0----model hyper-parameters for precision ($\beta_{gk}$)
%for example, m_0=0,  v_0=0.1,  a_0=20.0,  b_0=0.05
%%here we preprocess the data with mean 0 and std 1
%Outputs
%CMean_m---G x K matrix, optimal posterior mean parameters
%CAlpha---D x K  matrix, with each row vector as optimal posterior Dirichelet parameters
%Cfr_pro--- free energy vector verus process vector
%Cprocess---returned optimal process number


%--------------------------------------------------------------------------
clear all;
Path = 'C:\student\matlab\data\setlur\SetlurData\';
load([Path, 'setlurdata']);
 
%normalizing the datssets
[rns rnf] = size(x);
Data = normalize_meanstd(x);
%setting parameters for model
pro=[2:12];  %%%%%%%% vector of process
no_reps=200;  % number of EM iteration
Alphazero=1;  %prior Dirichlet  model parameter
% hyper-model parameters
m_0=0;
v_0=1;
a_0=100;
b_0=0.01;
mdisplay=1;  %%%%%%%%%% display  iteration detials
%--------------------------------------------------------------------------
Cfr_pro=[];                    %%% save free energy vector verus  process
if length(pro)>1
    for  K=pro(1):pro(end)
        Alpha_0=Alphazero*ones(1,K);
        % display iteration number wrt process
        fprintf('Check process: [%d]...\n',K);
        [CMean_m,CAlpha,CFenergy]...
            =cvb_pro(Data,K,no_reps,Alpha_0,m_0,v_0,a_0,b_0,mdisplay);
        Cfr_pro=[Cfr_pro CFenergy];
        [max_fr max_pro]=max(Cfr_pro);
        Cprocess=max_pro+pro(1)-1;
    end
else
    Cprocess = pro; % pro is fixed number not vector
end

%%%output the optimal mean and alpha for  the optimal process
fprintf('Clustering for optimal process [%d]\n', Cprocess);
[CMean_m,CAlpha,CFenergy]...
    =cvb_pro(Data,Cprocess,no_reps,Alphazero*ones(1,Cprocess),m_0,...
    v_0,a_0,b_0,mdisplay);
%plot the free energy vector verus process vector
figure;
plot( [pro(1):1:pro(end)], Cfr_pro,'*-'); axis([pro(1) pro(end) min(Cfr_pro)  0.98*max(Cfr_pro)]);
xlabel('process');
ylabel('free energy');
%plot the normalization of the optimal alpha (clusters)
CAlpha_norm=CAlpha./repmat(sum(CAlpha,2),1,Cprocess);
figure;
bar_plot(CAlpha_norm');
save([Path,'result_cvblpd_setlur.mat'], 'Cfr_pro', 'CMean_m', 'CAlpha', 'Cprocess');


