function [fr_pro,Mean_m,Alpha,process]=vb_vs_pro(Data,pro,mdisplay)



%%%%%%%%%%%%Initializing parameters
tol=0.001;
no_reps=100;
m_0=mean(mean(Data));
Alphazero=0.2;
v_0=1;
a_0=20;
b_0=0.05;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Input variables

%Data--- D x G (data x gene)  matrix of expression values
%pro--- process vector
%no_reps---number of iterations in the EM algorithm
%m_0,v_0---model hyper-parameters for mean ($\mu_{gk}$)
%a_0,b_0----model hyper-parameters for precision ($\beta_{gk}$)
%for example, m_0=0,  v_0=0.1,  a_0=20.0,  b_0=0.05  
%%here we preprocess the data with mean 0 and std 5. 
%tol-- tolerance for free energy increase
%Outputs
%Mean_m---G x K matrix, optimal posterior mean parameters  
%Alpha---D x K  matrix, with each row vector as optimal posterior Dirichelet parameters
%fr_pro--- free energy vector verus process vector 
%process---returned optimal process number

%%% Data is mean zero and std 5  corresponding to above model parameters 
no_pro=length(pro);
fr_pro=[];  %%%  free energy vector   verus  process
temp_fr=0;

for K=pro(1):pro(end)   
    Alpha_0=Alphazero*ones(1,K);  %%% fixed number of process, initialize the Alpha_0 
    %note the dimension is different 
 % display iteration number wrt process 
   if mdisplay==1
       v=sprintf('process: %d',K);
       disp(v);
   end
 [Mean_m, Alpha, Fenergy]...
     =vb_pro(Data,K,no_reps,tol,Alpha_0,m_0,v_0,a_0,b_0,mdisplay);
 fr_pro=[fr_pro Fenergy];  
end
[max_fr max_pro]=max(fr_pro);
process=max_pro+pro(1)-1;    %%%  optimal process based on free energy criterion 
%%%output the optimal mean and alpha of the optimal process 
[Mean_m,Alpha,Fenergy]...
    =vb_pro(Data,process,no_reps,tol,Alphazero*ones(1,process),m_0,v_0,a_0,b_0,mdisplay);
%plot the free energy vector verus process vector
figure; 
plot([pro(1):1:pro(end)],fr_pro,'*-'); axis([pro(1) pro(end) min(fr_pro)  0.98*max(fr_pro)]);
xlabel('process');
ylabel('free energy');
%plot the normalization of the optimal alpha (clusters)
Alpha_norm=Alpha./repmat(sum(Alpha,2),1,process);
figure;
bar_plot(Alpha_norm');
 

