function [Mean_m,Alpha,FE]...
    =vblpd(Data,K,no_reps,tol,KMean,Alpha_0,m_0,v_0,a_0,b_0,mdisplay)
%Input variables
%Data--- D x G (data x gene)  matrix of expression values
%K---number of processes
%no_reps---number of iterations in the algorithm
%Alpha_0 ---1 x  K, initial   Dirichlet hyperparameter
%m_0,v_0,a_0,b_0----initial hyperparameters
%for example, m_0=0,  v_0=0.1,  a_0=20.0,  b_0=0.05
%tol-- tolerance for free energy increase
%KMean----initializing the mean Mean_m, this is very critical

%Outputs
%Mean_m---G x K matrix, mean hyperparameters over means parameters
%Alpha---D x K matrix,  clustering posterior Dirichlet parameter
%FE--- free energy lower bound for evidence
%Initialize variables
[no_samp,no_feat]=size(Data);
% [Y C]=kmeans_cluster(Data,K);
%
% KMean=C';
%
% Mean_m=C';
Mean_m=KMean;
Mean_v=v_0*ones(no_feat,K);
Var_ga=a_0*ones(no_feat,K);
Var_gb=b_0*ones(no_feat,K);
rep=1;
Alpha=repmat(Alpha_0,no_samp,1);
Free_en=[];
temp_fr2=0;
change_er =1;
while ((rep<=no_reps) & change_er>=tol)       %  main iteration loop

    %updates for r_{dg,k}=P(Z_dg=k| gth_d), i.e., the probability of expression
    %value for gene g in sample d produced by process k

    %Caculating the summation of Alpha
    sa=sum(Alpha,2);
    temp_tgth=psi(Alpha)-psi(repmat(sa,1,K));     %%%% definition of log(\tilde(\theta)_{dk})
    temp_ab=Var_ga.*Var_gb;   %%%%%%%%%%%%%%%  a_{gk} b_{gk}
    %%%% updates for  r_{dg,k}, note that r_{dg,k} needs to be normalized w.r.t. the
    %%%% last index k
    R=(1/K).*ones(no_samp,no_feat,K);
    T=zeros(no_samp,no_feat,K);
    for d=1:no_samp
        temp1=(repmat(Data(d,:)',1,K)-Mean_m).^2;
        temp2=repmat(temp_tgth(d,:),no_feat,1)-0.5.*temp_ab.*(temp1+1./Mean_v)+0.5.*(psi(Var_ga)+log(Var_gb));
        T(d,:,:)=reshape(temp2,1,no_feat,K);
    end
    for d=1:no_samp
        temp1=reshape(-T(d,:,:),no_feat,K);
        M=mean(temp1,2);
        temp2=exp(-temp1+repmat(M,1,K));
        R(d,:,:)=reshape(temp2./repmat(sum(temp2,2),1,K),1,no_feat,K);
    end
    %updates for Alpha----alpha_{dk}
    %Alpha_0_exp=Alpha_0./sum(Alpha_0);  %average (expectation) of Alpha_0
    Alpha=repmat(Alpha_0,no_samp,1)+reshape(sum(R,2),no_samp,K);
    %updates for Mean_v---v_{gk}
    R_temp=reshape(sum(R,1),no_feat,K);  %%%%summation of R  over the first index
    Mean_v=v_0+temp_ab.*R_temp;
    %updates for Mean_m----m_{gk}
    for k=1:K
        Mean_m(:,k)=(1./Mean_v(:,k)).*(v_0*m_0+ temp_ab(:,k).*(sum(R(:,:,k).*Data))');
    end
    %updates for Var_ga and Var_gb------a_{gk},  b_{gk}
    Var_ga=a_0+0.5*reshape(sum(R,1),no_feat,K);
    for k=1:K
        temp_sum=reshape(R(:,:,k),no_samp,no_feat).*((Data-repmat(Mean_m(:,k)',no_samp,1)).^2);
        temp_gb=1/b_0+0.5*(sum(temp_sum)')+0.5*(1./Mean_v(:,k)).*R_temp(:,k);
        %%%here we use the summation of R over the first index:  R_temp
        Var_gb(:,k)=1./temp_gb;
    end
    temp_fr1=temp_fr2;
    temp_fr2=Free_energy(Data,Mean_m,Mean_v,R,Var_ga,Var_gb,Alpha_0,Alpha,m_0,v_0,a_0,b_0);
    change_er = abs(temp_fr2-temp_fr1)/abs(temp_fr2);
    if mdisplay==1
        v=sprintf('VB iteration [%d]: change of free energy %f ',rep,change_er);
        disp(v);
    end
    Free_en=[Free_en temp_fr2];
    rep=rep+1;
end

FE=Free_en(end);

% Alpha_norm=Alpha./repmat(sum(Alpha,2),1,K);
%
% figure;
% bar_plot(Alpha_norm');



